@@ -27,14 +27,6 @@
 #include "vm/method.h"
 #include "vm/call.h"
 
-unsigned long native_call(struct vm_method *method, const void *target,
-			  unsigned long *args)
+void native_call(struct vm_method *method, void *target, unsigned long *args, union jvalue *result)
 {
-	return 0;
-}
-
-unsigned long vm_native_call(struct vm_method *method, const void *target,
-			     unsigned long *args)
-{
-	return 0;
 }
